# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The buffervar commands module.

These commands are used in the following models:
SMU2450, SMU2460, SMU2461, SMU2470

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - bufferVar.capacity
    - bufferVar.clear()
    - bufferVar.dates[N]
    - bufferVar.endindex
    - bufferVar.extraformattedvalues[N]
    - bufferVar.extravalues[N]
    - bufferVar.extravalueunits[N]
    - bufferVar.fillmode
    - bufferVar.formattedreadings[N]
    - bufferVar.fractionalseconds[N]
    - bufferVar.logstate
    - bufferVar.n
    - bufferVar.readings[N]
    - bufferVar.relativetimestamps[N]
    - bufferVar.seconds[N]
    - bufferVar.sourceformattedvalues[N]
    - bufferVar.sourcestatuses[N]
    - bufferVar.sourceunits[N]
    - bufferVar.sourcevalues[N]
    - bufferVar.startindex
    - bufferVar.statuses[N]
    - bufferVar.times[N]
    - bufferVar.timestamps[N]
    - bufferVar.units[N]
"""
from typing import Dict, Optional, TYPE_CHECKING, Union

from .._helpers import BaseTSPCmd, DefaultDictDeviceCommunication, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


#  pylint: disable=too-many-instance-attributes,too-many-public-methods
class Buffervar(BaseTSPCmd):
    """The ``bufferVar`` command tree.

    **Info:**
        - ``bufferVar``, the name of the reading buffer, which may be a default buffer (defbuffer1
          or defbuffer2) or a user-defined buffer.

    Properties/methods:
        - ``.capacity``: The ``bufferVar.capacity`` attribute.
        - ``.clear()``: The ``bufferVar.clear()`` function.
        - ``.dates``: The ``bufferVar.dates[N]`` attribute.
        - ``.endindex``: The ``bufferVar.endindex`` attribute.
        - ``.extraformattedvalues``: The ``bufferVar.extraformattedvalues[N]`` attribute.
        - ``.extravalues``: The ``bufferVar.extravalues[N]`` attribute.
        - ``.extravalueunits``: The ``bufferVar.extravalueunits[N]`` attribute.
        - ``.fillmode``: The ``bufferVar.fillmode`` attribute.
        - ``.formattedreadings``: The ``bufferVar.formattedreadings[N]`` attribute.
        - ``.fractionalseconds``: The ``bufferVar.fractionalseconds[N]`` attribute.
        - ``.logstate``: The ``bufferVar.logstate`` attribute.
        - ``.n``: The ``bufferVar.n`` attribute.
        - ``.readings``: The ``bufferVar.readings[N]`` attribute.
        - ``.relativetimestamps``: The ``bufferVar.relativetimestamps[N]`` attribute.
        - ``.seconds``: The ``bufferVar.seconds[N]`` attribute.
        - ``.sourceformattedvalues``: The ``bufferVar.sourceformattedvalues[N]`` attribute.
        - ``.sourcestatuses``: The ``bufferVar.sourcestatuses[N]`` attribute.
        - ``.sourceunits``: The ``bufferVar.sourceunits[N]`` attribute.
        - ``.sourcevalues``: The ``bufferVar.sourcevalues[N]`` attribute.
        - ``.startindex``: The ``bufferVar.startindex`` attribute.
        - ``.statuses``: The ``bufferVar.statuses[N]`` attribute.
        - ``.times``: The ``bufferVar.times[N]`` attribute.
        - ``.timestamps``: The ``bufferVar.timestamps[N]`` attribute.
        - ``.units``: The ``bufferVar.units[N]`` attribute.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "bufferVar") -> None:
        super().__init__(device, cmd_syntax)
        self._dates: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.dates[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.dates[{{key}}])",
            device=self._device,
        )
        self._extraformattedvalues: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.extraformattedvalues[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.extraformattedvalues[{{key}}])",
            device=self._device,
        )
        self._extravalues: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.extravalues[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.extravalues[{{key}}])",
            device=self._device,
        )
        self._extravalueunits: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.extravalueunits[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.extravalueunits[{{key}}])",
            device=self._device,
        )
        self._formattedreadings: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.formattedreadings[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.formattedreadings[{{key}}])",
            device=self._device,
        )
        self._fractionalseconds: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.fractionalseconds[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.fractionalseconds[{{key}}])",
            device=self._device,
        )
        self._readings: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.readings[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.readings[{{key}}])",
            device=self._device,
        )
        self._relativetimestamps: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.relativetimestamps[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.relativetimestamps[{{key}}])",
            device=self._device,
        )
        self._seconds: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.seconds[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.seconds[{{key}}])",
            device=self._device,
        )
        self._sourceformattedvalues: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.sourceformattedvalues[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.sourceformattedvalues[{{key}}])",
            device=self._device,
        )
        self._sourcestatuses: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.sourcestatuses[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.sourcestatuses[{{key}}])",
            device=self._device,
        )
        self._sourceunits: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.sourceunits[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.sourceunits[{{key}}])",
            device=self._device,
        )
        self._sourcevalues: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.sourcevalues[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.sourcevalues[{{key}}])",
            device=self._device,
        )
        self._statuses: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.statuses[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.statuses[{{key}}])",
            device=self._device,
        )
        self._times: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.times[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.times[{{key}}])",
            device=self._device,
        )
        self._timestamps: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.timestamps[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.timestamps[{{key}}])",
            device=self._device,
        )
        self._units: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.units[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.units[{{key}}])",
            device=self._device,
        )

    @property
    def capacity(self) -> str:
        """Access the ``bufferVar.capacity`` attribute.

        **Description:**
            - This attribute sets the number of readings a buffer can store.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.capacity)`` query.
            - Setting this property to a value will send the ``bufferVar.capacity = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.capacity = value
            - print(bufferVar.capacity)

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".capacity"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.capacity)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.capacity`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @capacity.setter
    def capacity(self, value: Union[str, float]) -> None:
        """Access the ``bufferVar.capacity`` attribute.

        **Description:**
            - This attribute sets the number of readings a buffer can store.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.capacity)`` query.
            - Setting this property to a value will send the ``bufferVar.capacity = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.capacity = value
            - print(bufferVar.capacity)

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".capacity", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.capacity = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.capacity`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def dates(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.dates[N]`` attribute.

        **Description:**
            - This attribute contains the dates of readings that are stored in the reading buffer.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.dates[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.dates[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._dates

    @property
    def endindex(self) -> str:
        """Access the ``bufferVar.endindex`` attribute.

        **Description:**
            - This attribute indicates the last index in a reading buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.endindex)`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.endindex)

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".endindex"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.endindex)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.endindex`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def extraformattedvalues(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.extraformattedvalues[N]`` attribute.

        **Description:**
            - This attribute contains the measurement and the unit of measure of the additional
              values in a reading buffer.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.extraformattedvalues[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.extraformattedvalues[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._extraformattedvalues

    @property
    def extravalues(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.extravalues[N]`` attribute.

        **Description:**
            - This attribute contains the additional values in a reading buffer.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.extravalues[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.extravalues[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._extravalues

    @property
    def extravalueunits(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.extravalueunits[N]`` attribute.

        **Description:**
            - This attribute contains the units of the additional values in a reading buffer.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.extravalueunits[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.extravalueunits[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._extravalueunits

    @property
    def fillmode(self) -> str:
        """Access the ``bufferVar.fillmode`` attribute.

        **Description:**
            - This attribute determines if a reading buffer is filled continuously or is filled once
              and stops.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.fillmode)`` query.
            - Setting this property to a value will send the ``bufferVar.fillmode = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.fillmode = value
            - print(bufferVar.fillmode)

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".fillmode"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.fillmode)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.fillmode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @fillmode.setter
    def fillmode(self, value: Union[str, float]) -> None:
        """Access the ``bufferVar.fillmode`` attribute.

        **Description:**
            - This attribute determines if a reading buffer is filled continuously or is filled once
              and stops.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.fillmode)`` query.
            - Setting this property to a value will send the ``bufferVar.fillmode = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.fillmode = value
            - print(bufferVar.fillmode)

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".fillmode", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.fillmode = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.fillmode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def formattedreadings(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.formattedreadings[N]`` attribute.

        **Description:**
            - This attribute contains the stored readings shown as numbers with units and prefixes.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.formattedreadings[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.formattedreadings[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._formattedreadings

    @property
    def fractionalseconds(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.fractionalseconds[N]`` attribute.

        **Description:**
            - This attribute contains the fractional second portion of the timestamp of each reading
              in the reading buffer.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.fractionalseconds[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.fractionalseconds[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._fractionalseconds

    @property
    def logstate(self) -> str:
        """Access the ``bufferVar.logstate`` attribute.

        **Description:**
            - This attribute indicates if information events are logged when the specified reading
              buffer is at 0% or 100% filled.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.logstate)`` query.
            - Setting this property to a value will send the ``bufferVar.logstate = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.logstate = value
            - print(bufferVar.logstate)

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".logstate"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.logstate)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.logstate`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @logstate.setter
    def logstate(self, value: Union[str, float]) -> None:
        """Access the ``bufferVar.logstate`` attribute.

        **Description:**
            - This attribute indicates if information events are logged when the specified reading
              buffer is at 0% or 100% filled.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.logstate)`` query.
            - Setting this property to a value will send the ``bufferVar.logstate = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.logstate = value
            - print(bufferVar.logstate)

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".logstate", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.logstate = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.logstate`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def n(self) -> str:
        """Access the ``bufferVar.n`` attribute.

        **Description:**
            - This attribute contains the number of readings in the specified reading buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.n)`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.n)

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".n"
            return self._device.query(f"print({self._cmd_syntax}.n)")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.n`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def readings(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.readings[N]`` attribute.

        **Description:**
            - This attribute contains the readings stored in a specified reading buffer.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.readings[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.readings[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._readings

    @property
    def relativetimestamps(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.relativetimestamps[N]`` attribute.

        **Description:**
            - This attribute contains the timestamps, in seconds, when each reading occurred,
              relative to the timestamp of the first entry in the reading buffer.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.relativetimestamps[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.relativetimestamps[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._relativetimestamps

    @property
    def seconds(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.seconds[N]`` attribute.

        **Description:**
            - This attribute contains the timestamp of a reading in seconds, in UTC format.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.seconds[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.seconds[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._seconds

    @property
    def sourceformattedvalues(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.sourceformattedvalues[N]`` attribute.

        **Description:**
            - This attribute contains the source levels formatted as they appear on the front-panel
              display when the readings in the reading buffer were acquired.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.sourceformattedvalues[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.sourceformattedvalues[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._sourceformattedvalues

    @property
    def sourcestatuses(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.sourcestatuses[N]`` attribute.

        **Description:**
            - This attribute contains the source status conditions of the instrument for the reading
              point.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.sourcestatuses[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.sourcestatuses[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._sourcestatuses

    @property
    def sourceunits(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.sourceunits[N]`` attribute.

        **Description:**
            - This attribute contains the units of measure of the source.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.sourceunits[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.sourceunits[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._sourceunits

    @property
    def sourcevalues(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.sourcevalues[N]`` attribute.

        **Description:**
            - This attribute contains the source levels being output when readings in the reading
              buffer were acquired.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.sourcevalues[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.sourcevalues[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._sourcevalues

    @property
    def startindex(self) -> str:
        """Access the ``bufferVar.startindex`` attribute.

        **Description:**
            - This attribute indicates the starting index in a reading buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.startindex)`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.startindex)

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".startindex"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.startindex)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.startindex`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def statuses(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.statuses[N]`` attribute.

        **Description:**
            - This attribute contains the status values of readings in the reading buffer.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.statuses[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.statuses[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._statuses

    @property
    def times(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.times[N]`` attribute.

        **Description:**
            - This attribute contains the time when the instrument made the reading.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.times[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.times[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._times

    @property
    def timestamps(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.timestamps[N]`` attribute.

        **Description:**
            - This attribute contains the timestamp when each reading saved in the specified reading
              buffer occurred.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.timestamps[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.timestamps[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._timestamps

    @property
    def units(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.units[N]`` attribute.

        **Description:**
            - This attribute contains the unit of measure that is stored with readings in the
              reading buffer.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.units[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.units[N])

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._units

    def clear(self) -> None:
        """Run the ``bufferVar.clear()`` function.

        **Description:**
            - This function clears all readings and statistics from the specified buffer.

        **TSP Syntax:**

        ::

            - bufferVar.clear()

        **Info:**
            - ``bufferVar``, the name of the reading buffer, which may be a default buffer
              (defbuffer1 or defbuffer2) or a user-defined buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The beeper commands module.

These commands are used in the following models:
SMU2601B, SMU2602B, SMU2604B, SMU2606B, SMU2611B, SMU2612B, SMU2614B, SMU2634B, SMU2635B, SMU2636B,
SMU2651A, SMU2657A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - beeper.beep()
    - beeper.enable
"""
from typing import Optional, TYPE_CHECKING, Union

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class Beeper(BaseTSPCmd):
    """The ``beeper`` command tree.

    Constants:
        - ``.OFF``: This command turns the beeper off.
        - ``.ON``: This command turns the beeper on.

    Properties/methods:
        - ``.beep()``: The ``beeper.beep()`` function.
        - ``.enable``: The ``beeper.enable`` attribute.
    """

    OFF = "beeper.OFF"
    """str: This command turns the beeper off."""
    ON = "beeper.ON"
    """str: This command turns the beeper on."""

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "beeper") -> None:
        super().__init__(device, cmd_syntax)

    @property
    def enable(self) -> str:
        """Access the ``beeper.enable`` attribute.

        **Description:**
            - This command allows you to turn the beeper on or off.

        **Usage:**
            - Accessing this property will send the ``print(beeper.enable)`` query.
            - Setting this property to a value will send the ``beeper.enable = value`` command.

        **TSP Syntax:**

        ::

            - beeper.enable = value
            - print(beeper.enable)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".enable"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.enable)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @enable.setter
    def enable(self, value: Union[str, float]) -> None:
        """Access the ``beeper.enable`` attribute.

        **Description:**
            - This command allows you to turn the beeper on or off.

        **Usage:**
            - Accessing this property will send the ``print(beeper.enable)`` query.
            - Setting this property to a value will send the ``beeper.enable = value`` command.

        **TSP Syntax:**

        ::

            - beeper.enable = value
            - print(beeper.enable)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".enable", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.enable = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def beep(self, duration: float, frequency: float) -> None:
        """Run the ``beeper.beep()`` function.

        **Description:**
            - This function generates an audible tone.

        **TSP Syntax:**

        ::

            - beeper.beep()

        Args:
            duration: The amount of time to play the tone (0.001 s to 100 s).
            frequency: The frequency of the tone in Hertz (Hz).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.beep({duration}, {frequency})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.beep()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



# pylint: disable=too-many-lines
# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The channel commands module.

These commands are used in the following models:
DAQ6510

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - channel.close()
    - channel.getclose()
    - channel.getcommonside()
    - channel.getcount()
    - channel.getcountinterval()
    - channel.getdelay()
    - channel.getdmm()
    - channel.getlabel()
    - channel.getmatch()
    - channel.getmatchtype()
    - channel.getmode()
    - channel.getstate()
    - channel.gettype()
    - channel.getwidth()
    - channel.multiple.close()
    - channel.multiple.open()
    - channel.open()
    - channel.read()
    - channel.setcommonside()
    - channel.setcountinterval()
    - channel.setdelay()
    - channel.setdmm()
    - channel.setlabel()
    - channel.setmatch()
    - channel.setmatchtype()
    - channel.setmode()
    - channel.setwidth()
    - channel.write()
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class ChannelMultiple(BaseTSPCmd):
    """The ``channel.multiple`` command tree.

    Properties/methods:
        - ``.close()``: The ``channel.multiple.close()`` function.
        - ``.open()``: The ``channel.multiple.open()`` function.
    """

    def close(self, channel_list: str) -> None:
        """Run the ``channel.multiple.close()`` function.

        **Description:**
            - This function closes the listed channels without affecting any other channels.

        **TSP Syntax:**

        ::

            - channel.multiple.close()

        Args:
            channel_list: The list of channels to close.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.close("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.close()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def open(self, channel_list: str) -> None:
        """Run the ``channel.multiple.open()`` function.

        **Description:**
            - This function opens the channels in the channel list without affecting any others.

        **TSP Syntax:**

        ::

            - channel.multiple.open()

        Args:
            channel_list: A list of the channels to open.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.open("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.open()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


#  pylint: disable=too-many-public-methods
class Channel(BaseTSPCmd):
    """The ``channel`` command tree.

    Constants:
        - ``.COUNT_INTERVAL_10M``: The storage of channel relay closure count is set to 10 minutes.
        - ``.COUNT_INTERVAL_15M``: The storage of channel relay closure count is set to 15 minutes.
        - ``.COUNT_INTERVAL_1M``: The storage of channel relay closure count is set to 1 minute.
        - ``.COUNT_INTERVAL_24H``: The storage of channel relay closure count is set to 24 hours.
        - ``.COUNT_INTERVAL_30M``: The storage of channel relay closure count is set to 30 minutes.
        - ``.COUNT_INTERVAL_5M``: The storage of channel relay closure count is set to 5 minutes.
        - ``.COUNT_INTERVAL_60M``: The storage of channel relay closure count is set to 60 minutes.
        - ``.IND_CLOSED``: Channel is closed.
        - ``.IND_MATCH``: Digital I/O or totalizer channel value is matched.
        - ``.IND_OVERFLOW``: Totalizer channel has overflowed.
        - ``.MATCH_ANY``: For an any match, a match is when the match value equals the channel-read
          value.
        - ``.MATCH_EXACT``: The state match indicator only becomes true when the match value equals
          the channel-read value.
        - ``.MATCH_NONE``: When none is set, matching is disabled.
        - ``.MATCH_UNCHANGED``: For an unchanged match, the value should be the same as the
          original. If not, a match is declared.
        - ``.MODE_FALLING_EDGE``: For totalizer channels, select falling edge.
        - ``.MODE_FALLING_EDGE_READ_RESET``: For totalizer channels, select falling edge, then reset
          the count on read.
        - ``.MODE_INPUT``: Set digital channel to input.
        - ``.MODE_OUTPUT``: Set digital channel to output.
        - ``.MODE_RISING_EDGE``: For totalizer channels, select rising edge.
        - ``.MODE_RISING_EDGE_READ_RESET``: For totalizer channels, select rising edge, then reset
          the count on read.
        - ``.TYPE_BACKPLANE``: Backplane.
        - ``.TYPE_DAC``: DAC channel.
        - ``.TYPE_DIGITAL``: Digital channel.
        - ``.TYPE_POLE``: Pole channel.
        - ``.TYPE_RADIO``: Radio channel.
        - ``.TYPE_SWITCH``: Switch channel.
        - ``.TYPE_TOTALIZER``: Totalizer channel.

    Properties/methods:
        - ``.close()``: The ``channel.close()`` function.
        - ``.getclose()``: The ``channel.getclose()`` function.
        - ``.getcommonside()``: The ``channel.getcommonside()`` function.
        - ``.getcount()``: The ``channel.getcount()`` function.
        - ``.getcountinterval()``: The ``channel.getcountinterval()`` function.
        - ``.getdelay()``: The ``channel.getdelay()`` function.
        - ``.getdmm()``: The ``channel.getdmm()`` function.
        - ``.getlabel()``: The ``channel.getlabel()`` function.
        - ``.getmatch()``: The ``channel.getmatch()`` function.
        - ``.getmatchtype()``: The ``channel.getmatchtype()`` function.
        - ``.getmode()``: The ``channel.getmode()`` function.
        - ``.getstate()``: The ``channel.getstate()`` function.
        - ``.gettype()``: The ``channel.gettype()`` function.
        - ``.getwidth()``: The ``channel.getwidth()`` function.
        - ``.multiple``: The ``channel.multiple`` command tree.
        - ``.open()``: The ``channel.open()`` function.
        - ``.read()``: The ``channel.read()`` function.
        - ``.setcommonside()``: The ``channel.setcommonside()`` function.
        - ``.setcountinterval()``: The ``channel.setcountinterval()`` function.
        - ``.setdelay()``: The ``channel.setdelay()`` function.
        - ``.setdmm()``: The ``channel.setdmm()`` function.
        - ``.setlabel()``: The ``channel.setlabel()`` function.
        - ``.setmatch()``: The ``channel.setmatch()`` function.
        - ``.setmatchtype()``: The ``channel.setmatchtype()`` function.
        - ``.setmode()``: The ``channel.setmode()`` function.
        - ``.setwidth()``: The ``channel.setwidth()`` function.
        - ``.write()``: The ``channel.write()`` function.
    """

    COUNT_INTERVAL_10M = "channel.COUNT_INTERVAL_10M"
    """str: The storage of channel relay closure count is set to 10 minutes."""
    COUNT_INTERVAL_15M = "channel.COUNT_INTERVAL_15M"
    """str: The storage of channel relay closure count is set to 15 minutes."""
    COUNT_INTERVAL_1M = "channel.COUNT_INTERVAL_1M"
    """str: The storage of channel relay closure count is set to 1 minute."""
    COUNT_INTERVAL_24H = "channel.COUNT_INTERVAL_24H"
    """str: The storage of channel relay closure count is set to 24 hours."""
    COUNT_INTERVAL_30M = "channel.COUNT_INTERVAL_30M"
    """str: The storage of channel relay closure count is set to 30 minutes."""
    COUNT_INTERVAL_5M = "channel.COUNT_INTERVAL_5M"
    """str: The storage of channel relay closure count is set to 5 minutes."""
    COUNT_INTERVAL_60M = "channel.COUNT_INTERVAL_60M"
    """str: The storage of channel relay closure count is set to 60 minutes."""
    IND_CLOSED = "channel.IND_CLOSED"
    """str: Channel is closed."""
    IND_MATCH = "channel.IND_MATCH"
    """str: Digital I/O or totalizer channel value is matched."""
    IND_OVERFLOW = "channel.IND_OVERFLOW"
    """str: Totalizer channel has overflowed."""
    MATCH_ANY = "channel.MATCH_ANY"
    """str: For an any match, a match is when the match value equals the channel-read value."""
    MATCH_EXACT = "channel.MATCH_EXACT"
    """str: The state match indicator only becomes true when the match value equals the channel-read value."""  # noqa: E501
    MATCH_NONE = "channel.MATCH_NONE"
    """str: When none is set, matching is disabled."""
    MATCH_UNCHANGED = "channel.MATCH_UNCHANGED"
    """str: For an unchanged match, the value should be the same as the original. If not, a match is declared."""  # noqa: E501
    MODE_FALLING_EDGE = "channel.MODE_FALLING_EDGE"
    """str: For totalizer channels, select falling edge."""
    MODE_FALLING_EDGE_READ_RESET = "channel.MODE_FALLING_EDGE_READ_RESET"
    """str: For totalizer channels, select falling edge, then reset the count on read."""
    MODE_INPUT = "channel.MODE_INPUT"
    """str: Set digital channel to input."""
    MODE_OUTPUT = "channel.MODE_OUTPUT"
    """str: Set digital channel to output."""
    MODE_RISING_EDGE = "channel.MODE_RISING_EDGE"
    """str: For totalizer channels, select rising edge."""
    MODE_RISING_EDGE_READ_RESET = "channel.MODE_RISING_EDGE_READ_RESET"
    """str: For totalizer channels, select rising edge, then reset the count on read."""
    TYPE_BACKPLANE = "channel.TYPE_BACKPLANE"
    """str: Backplane."""
    TYPE_DAC = "channel.TYPE_DAC"
    """str: DAC channel."""
    TYPE_DIGITAL = "channel.TYPE_DIGITAL"
    """str: Digital channel."""
    TYPE_POLE = "channel.TYPE_POLE"
    """str: Pole channel."""
    TYPE_RADIO = "channel.TYPE_RADIO"
    """str: Radio channel."""
    TYPE_SWITCH = "channel.TYPE_SWITCH"
    """str: Switch channel."""
    TYPE_TOTALIZER = "channel.TYPE_TOTALIZER"
    """str: Totalizer channel."""

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "channel") -> None:
        super().__init__(device, cmd_syntax)
        self._multiple = ChannelMultiple(device, f"{self._cmd_syntax}.multiple")

    @property
    def multiple(self) -> ChannelMultiple:
        """Return the ``channel.multiple`` command tree.

        Sub-properties/methods:
            - ``.close()``: The ``channel.multiple.close()`` function.
            - ``.open()``: The ``channel.multiple.open()`` function.
        """
        return self._multiple

    def close(self, channel_list: str) -> None:
        """Run the ``channel.close()`` function.

        **Description:**
            - This function closes the channels and channel pairs that are specified by the channel
              list parameter.

        **TSP Syntax:**

        ::

            - channel.close()

        Args:
            channel_list: A string that lists the channels and channel pairs to close.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.close("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.close()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getclose(self, channel_list: Optional[str] = None) -> str:
        """Run the ``channel.getclose()`` function.

        **Description:**
            - This function queries for the closed channels indicated by the channel list parameter.

        **TSP Syntax:**

        ::

            - channel.getclose()

        Args:
            channel_list (optional): A string that lists the channels to be queried (including
                allslots, slot1, or slot2 to get information on all channels in both slots or a
                specific slot); returns all slots if nothing is specified.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (f'"{channel_list}"' if channel_list is not None else None,)
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.getclose({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getclose()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getcommonside(self, slot: str) -> str:
        """Run the ``channel.getcommonside()`` function.

        **Description:**
            - This function lists the commonside setting for the specified slot.

        **TSP Syntax:**

        ::

            - channel.getcommonside()

        Args:
            slot: A string that lists the slot number; can be allslots, slot1, or slot2.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getcommonside("{slot}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getcommonside()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getcount(self, channel_list: str) -> str:
        """Run the ``channel.getcount()`` function.

        **Description:**
            - This function returns the number of times the relays have been closed for the
              specified channels.

        **TSP Syntax:**

        ::

            - channel.getcount()

        Args:
            channel_list: A string listing the items to query, which can include.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getcount("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getcount()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getcountinterval(self) -> str:
        """Run the ``channel.getcountinterval()`` function.

        **Description:**
            - This function returns the interval that is presently set for storage of channel relay
              closure counts.

        **TSP Syntax:**

        ::

            - channel.getcountinterval()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.getcountinterval())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getcountinterval()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getdelay(self, channel_list: str) -> str:
        """Run the ``channel.getdelay()`` function.

        **Description:**
            - This function queries for the additional delay time for the specified channels.

        **TSP Syntax:**

        ::

            - channel.getdelay()

        Args:
            channel_list: A string listing the channels to query for their delay times; slot1,
                slot2, or allslots allowed.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getdelay("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getdelay()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getdmm(self, channel_list: str, setting: str) -> str:
        """Run the ``channel.getdmm()`` function.

        **Description:**
            - This function returns the setting for a channel DMM attribute.

        **TSP Syntax:**

        ::

            - channel.getdmm()

        Args:
            channel_list: List of channels to get from the DMM.
            setting: The DMM function or the parameter values to return for the selected channels.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getdmm("{channel_list}", {setting}))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getdmm()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getlabel(self, channel_number: str) -> str:
        """Run the ``channel.getlabel()`` function.

        **Description:**
            - This function returns the label associated with the specified channel.

        **TSP Syntax:**

        ::

            - channel.getlabel()

        Args:
            channel_number: A string that lists the channel to query for the associated label.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getlabel("{channel_number}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getlabel()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getmatch(self, channel_list: str) -> str:
        """Run the ``channel.getmatch()`` function.

        **Description:**
            - This function gets the match value on a channel.

        **TSP Syntax:**

        ::

            - channel.getmatch()

        Args:
            channel_list: String specifying digital I/O or totalizer channels to query, using normal
                channel list syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getmatch("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getmatch()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getmatchtype(self, channel_list: str) -> str:
        """Run the ``channel.getmatchtype()`` function.

        **Description:**
            - This function returns the match type for digital I/O and totalizer channels.

        **TSP Syntax:**

        ::

            - channel.getmatchtype()

        Args:
            channel_list: String specifying the digital I/O or totalizer channels to query, using
                normal channel list syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getmatchtype("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getmatchtype()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getmode(self, channel_list: str) -> str:
        """Run the ``channel.getmode()`` function.

        **Description:**
            - This function returns the present mode of digital and totalizer channels.

        **TSP Syntax:**

        ::

            - channel.getmode()

        Args:
            channel_list: String that specifies the channels to query, using normal channel list
                syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getmode("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getmode()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getstate(self, channel_list: Optional[str] = None) -> str:
        """Run the ``channel.getstate()`` function.

        **Description:**
            - This function returns the state indicators of the channels in the instrument.

        **TSP Syntax:**

        ::

            - channel.getstate()

        Args:
            channel_list (optional): String specifying the channels to query; use normal channel
                list syntax; if no channels are defined, all slots are returned.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (f'"{channel_list}"' if channel_list is not None else None,)
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.getstate({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getstate()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def gettype(self, channel_list: str) -> str:
        """Run the ``channel.gettype()`` function.

        **Description:**
            - This function returns the type associated with a channel.

        **TSP Syntax:**

        ::

            - channel.gettype()

        Args:
            channel_list: String specifying the channels to query, using normal channelList syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.gettype("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.gettype()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getwidth(self, channel_number: int) -> str:
        """Run the ``channel.getwidth()`` function.

        **Description:**
            - This function gets the width used by the channel.

        **TSP Syntax:**

        ::

            - channel.getwidth()

        Args:
            channel_number: The first channel number of the channels to be combined.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getwidth("{channel_number}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getwidth()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def open(self, channel_list: str) -> None:
        """Run the ``channel.open()`` function.

        **Description:**
            - This command opens the specified channels and channel pairs.

        **TSP Syntax:**

        ::

            - channel.open()

        Args:
            channel_list: String listing the channels to open; allslots, slot1, and slot2 available,
                which open all channels on all slots, all channels on slot 1, or all channels on
                slot 2.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.open("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.open()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def read(self, channel_list: str, reading_buffer: Optional[str] = None) -> str:
        """Run the ``channel.read()`` function.

        **Description:**
            - This function reads a value from a totalizer, DAC, or digital I/O channel.

        **TSP Syntax:**

        ::

            - channel.read()

        Args:
            channel_list: String that specifies a list of channels, using channel list notation.
            reading_buffer (optional): Reading buffer in which to store read values.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    reading_buffer,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.read({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.read()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setcommonside(self, slot: str, state: str) -> None:
        """Run the ``channel.setcommonside()`` function.

        **Description:**
            - This function sets up the specified slot to have a common side path.

        **TSP Syntax:**

        ::

            - channel.setcommonside()

        Args:
            slot: A string that lists the slot number; can be allslots, slot1, or slot2.
            state: channel.ON or channel.OFF.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setcommonside("{slot}", {state})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setcommonside()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setcountinterval(self, interval: str) -> None:
        """Run the ``channel.setcountinterval()`` function.

        **Description:**
            - This function determines how often the instrument stores the channel relay closure
              count.

        **TSP Syntax:**

        ::

            - channel.setcountinterval()

        Args:
            interval: The count update interval.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.setcountinterval({interval})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setcountinterval()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setdelay(self, channel_list: str, delay: float) -> None:
        """Run the ``channel.setdelay()`` function.

        **Description:**
            - This function sets additional delay time for specified channels.

        **TSP Syntax:**

        ::

            - channel.setdelay()

        Args:
            channel_list: A string listing the channels for which to add delay time.
            delay: Delay time for the selected channels; minimum is 0 seconds.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setdelay("{channel_list}", {delay})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setdelay()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setdmm(self, channel_list: str, setting: str, value: str) -> None:
        """Run the ``channel.setdmm()`` function.

        **Description:**
            - This function configures the DMM for a channel or group of channels.

        **TSP Syntax:**

        ::

            - channel.setdmm()

        Args:
            channel_list: List of channels for which to set the DMM.
            setting: The DMM function or the parameter for the function to be applied to the
                channelList; refer to Details and the tables following the examples.
            value: The function or attribute value.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setdmm("{channel_list}", {setting}, {value})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setdmm()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setlabel(self, channel_number: str, labelname: str) -> None:
        """Run the ``channel.setlabel()`` function.

        **Description:**
            - This function sets the label associated with a channel.

        **TSP Syntax:**

        ::

            - channel.setlabel()

        Args:
            channel_number: A string that lists the channel for which to set the label.
            labelname: A string that contains the label for the channel in channelNumber, up to 19
                characters.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setlabel("{channel_number}", "{labelname}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setlabel()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setmatch(self, channel_list: str, match_value: str, mask: Optional[str] = None) -> None:
        """Run the ``channel.setmatch()`` function.

        **Description:**
            - This function sets the match value on a digital input or totalizer channel.

        **TSP Syntax:**

        ::

            - channel.setmatch()

        Args:
            channel_list: String that specifies the channels to query, using normal channel list
                syntax.
            match_value: Channel value to compare on the specified channel.
            mask (optional): Value to specify the bits used to mask matchValue.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    match_value,
                    mask,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.setmatch({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setmatch()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setmatchtype(self, channel_list: str, type_: str) -> None:
        """Run the ``channel.setmatchtype()`` function.

        **Description:**
            - This function sets the match type on a digital I/O or totalizer channel.

        **TSP Syntax:**

        ::

            - channel.setmatchtype()

        Args:
            channel_list: String specifying the channels to set, using normal channel list syntax.
            type_: A value for setting the match operation used on this specific channel.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setmatchtype("{channel_list}", {type_})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setmatchtype()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setmode(self, channel_list: str, mode: str) -> None:
        """Run the ``channel.setmode()`` function.

        **Description:**
            - This function sets the mode of operation of a channel.

        **TSP Syntax:**

        ::

            - channel.setmode()

        Args:
            channel_list: String specifying the channels to set, using normal channel list syntax.
            mode: The mode of operation for a channel; for digital channels, the options are.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setmode("{channel_list}", {mode})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setmode()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setwidth(self, channel_number: int, width: int) -> None:
        """Run the ``channel.setwidth()`` function.

        **Description:**
            - This function allows you to control multiple digital I/O channels as one channel.

        **TSP Syntax:**

        ::

            - channel.setwidth()

        Args:
            channel_number: The first channel number of the channels to be combined.
            width: The number of channels to combine.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setwidth("{channel_number}", {width})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setwidth()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def write(self, channel_list: str, value: str) -> None:
        """Run the ``channel.write()`` function.

        **Description:**
            - This function writes a value to a channel.

        **TSP Syntax:**

        ::

            - channel.write()

        Args:
            channel_list: The channels to set, using standard channel naming.
            value: The value to be written to the channel (must be decimal value).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.write("{channel_list}", {value})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.write()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The instrument commands module.

These commands are used in the following models:
AWG70KA, AWG70KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - INSTrument:COUPle:SOURce {0|1|OFF|ON}
    - INSTrument:COUPle:SOURce?
    - INSTrument:MODE {AWG|FGEN}
    - INSTrument:MODE?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class InstrumentMode(SCPICmdWrite, SCPICmdRead):
    """The ``INSTrument:MODE`` command.

    **Description:**
        - This command sets or returns the AWG mode, either the AWG mode or the Function generator
          mode.

    **Usage:**
        - Using the ``.query()`` method will send the ``INSTrument:MODE?`` query.
        - Using the ``.verify(value)`` method will send the ``INSTrument:MODE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``INSTrument:MODE value`` command.

    **SCPI Syntax:**

    ::

        - INSTrument:MODE {AWG|FGEN}
        - INSTrument:MODE?

    **Info:**
        - ``AWG`` sets the instrument to the Arbitrary Waveform Generator mode. FGEN sets the
          instrument to the Function generator mode.
        - ``*RST`` sets this to AWG.
    """


class InstrumentCoupleSource(SCPICmdWrite, SCPICmdRead):
    """The ``INSTrument:COUPle:SOURce`` command.

    **Description:**
        - This command sets or returns the coupled state of the channel's Analog and Marker output
          controls of multi channel instruments. The set form of this command forces channel 2 to
          match channel 1. After the initial coupling of the settings, changes made to either
          channel 1 or channel 2 amplitude settings affect both channels.

    **Usage:**
        - Using the ``.query()`` method will send the ``INSTrument:COUPle:SOURce?`` query.
        - Using the ``.verify(value)`` method will send the ``INSTrument:COUPle:SOURce?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``INSTrument:COUPle:SOURce value``
          command.

    **SCPI Syntax:**

    ::

        - INSTrument:COUPle:SOURce {0|1|OFF|ON}
        - INSTrument:COUPle:SOURce?

    **Info:**
        - ``*RST`` sets this to 0.
    """


class InstrumentCouple(SCPICmdRead):
    """The ``INSTrument:COUPle`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``INSTrument:COUPle?`` query.
        - Using the ``.verify(value)`` method will send the ``INSTrument:COUPle?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.source``: The ``INSTrument:COUPle:SOURce`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._source = InstrumentCoupleSource(device, f"{self._cmd_syntax}:SOURce")

    @property
    def source(self) -> InstrumentCoupleSource:
        """Return the ``INSTrument:COUPle:SOURce`` command.

        **Description:**
            - This command sets or returns the coupled state of the channel's Analog and Marker
              output controls of multi channel instruments. The set form of this command forces
              channel 2 to match channel 1. After the initial coupling of the settings, changes made
              to either channel 1 or channel 2 amplitude settings affect both channels.

        **Usage:**
            - Using the ``.query()`` method will send the ``INSTrument:COUPle:SOURce?`` query.
            - Using the ``.verify(value)`` method will send the ``INSTrument:COUPle:SOURce?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``INSTrument:COUPle:SOURce value``
              command.

        **SCPI Syntax:**

        ::

            - INSTrument:COUPle:SOURce {0|1|OFF|ON}
            - INSTrument:COUPle:SOURce?

        **Info:**
            - ``*RST`` sets this to 0.
        """
        return self._source


class Instrument(SCPICmdRead):
    """The ``INSTrument`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``INSTrument?`` query.
        - Using the ``.verify(value)`` method will send the ``INSTrument?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.couple``: The ``INSTrument:COUPle`` command tree.
        - ``.mode``: The ``INSTrument:MODE`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "INSTrument") -> None:
        super().__init__(device, cmd_syntax)
        self._couple = InstrumentCouple(device, f"{self._cmd_syntax}:COUPle")
        self._mode = InstrumentMode(device, f"{self._cmd_syntax}:MODE")

    @property
    def couple(self) -> InstrumentCouple:
        """Return the ``INSTrument:COUPle`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``INSTrument:COUPle?`` query.
            - Using the ``.verify(value)`` method will send the ``INSTrument:COUPle?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.source``: The ``INSTrument:COUPle:SOURce`` command.
        """
        return self._couple

    @property
    def mode(self) -> InstrumentMode:
        """Return the ``INSTrument:MODE`` command.

        **Description:**
            - This command sets or returns the AWG mode, either the AWG mode or the Function
              generator mode.

        **Usage:**
            - Using the ``.query()`` method will send the ``INSTrument:MODE?`` query.
            - Using the ``.verify(value)`` method will send the ``INSTrument:MODE?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``INSTrument:MODE value`` command.

        **SCPI Syntax:**

        ::

            - INSTrument:MODE {AWG|FGEN}
            - INSTrument:MODE?

        **Info:**
            - ``AWG`` sets the instrument to the Arbitrary Waveform Generator mode. FGEN sets the
              instrument to the Function generator mode.
            - ``*RST`` sets this to AWG.
        """
        return self._mode



"""The setup_1 commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - SETUp:NAMe <NR1>,<QString>? <NR1>
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class SetupName(SCPICmdWrite):
    """The ``SETUp:NAMe`` command.

    **Description:**
        - This command allows you to create (or query) a name for your saved setups. The default
          name for all user setups is 'User.' The default name for factory setups is 'Factory.' This
          command is equivalent to selecting Save As from the File menu, pressing the Setup button,
          selecting the desired setup location, clicking the keyboard icon, and entering your setup
          name.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SETUp:NAMe value`` command.

    **SCPI Syntax:**

    ::

        - SETUp:NAMe <NR1>,<QString>? <NR1>

    **Info:**
        - ``<NR1>`` specifies a location in which the setup label is stored. Location values range
          from 1 through 10.
        - ``<QString>`` is a string containing the setup label.
    """


class Setup(SCPICmdRead):
    """The ``SETUp`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SETUp?`` query.
        - Using the ``.verify(value)`` method will send the ``SETUp?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.name``: The ``SETUp:NAMe`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SETUp") -> None:
        super().__init__(device, cmd_syntax)
        self._name = SetupName(device, f"{self._cmd_syntax}:NAMe")

    @property
    def name(self) -> SetupName:
        """Return the ``SETUp:NAMe`` command.

        **Description:**
            - This command allows you to create (or query) a name for your saved setups. The default
              name for all user setups is 'User.' The default name for factory setups is 'Factory.'
              This command is equivalent to selecting Save As from the File menu, pressing the Setup
              button, selecting the desired setup location, clicking the keyboard icon, and entering
              your setup name.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SETUp:NAMe value`` command.

        **SCPI Syntax:**

        ::

            - SETUp:NAMe <NR1>,<QString>? <NR1>

        **Info:**
            - ``<NR1>`` specifies a location in which the setup label is stored. Location values
              range from 1 through 10.
            - ``<QString>`` is a string containing the setup label.
        """
        return self._name



"""The AWG7KC commands module.

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""
from typing import Any, Dict, Optional

from tm_devices.drivers.pi.pi_device import PIDevice

from ._5y90wz_lpdmsomdodpoafgawgdsa.status_and_error import Opt
from ._5ylo0x_lpdmsodpomdoawgdsa.status_and_error import Ese, Sre
from ._5z14a1_lpdmsodpomdoafgawgdsa.calibration import Cal
from ._5z14a1_lpdmsodpomdoafgawgdsa.miscellaneous import Idn, Trg, Tst
from ._5z14a1_lpdmsodpomdoafgawgdsa.status_and_error import Cls, Esr, Opc, Rst, Stb, Wai
from ._32dszm_awg.awgcontrol import Awgcontrol
from ._32dszm_awg.diagnostic import Diagnostic
from ._32dszm_awg.display import Display
from ._32dszm_awg.event import Event
from ._32dszm_awg.instrument import Instrument
from ._32dszm_awg.mmemory import Mmemory
from ._32dszm_awg.output import OutputItem
from ._32dszm_awg.sequence import Sequence
from ._32dszm_awg.slist import Slist
from ._32dszm_awg.source import SourceItem
from ._32dszm_awg.status import Status
from ._32dszm_awg.system import System
from ._32dszm_awg.trigger import Trigger
from ._32dszm_awg.wlist import Wlist
from ._33ijgq_afgawg.abort import Abort
from ._33ijgq_afgawg.calibration import Calibration
from ._helpers import DefaultDictPassKeyToFactory


# pylint: disable=too-few-public-methods
class AWG7KCCommandConstants:
    """The AWG7KC command argument constants.

    This provides access to all the string constants which can be used as arguments for AWG7KC
    commands.
    """

    CONTINUOUS = "CONTINUOUS"  # CONTinuous
    ENHANCED = "ENHANCED"  # ENHanced
    EVENT_PATTERN = "EVENT_PATTERN"  # event_pattern
    FIRST = "FIRST"  # FIRSt
    FIXED = "FIXED"  # FIXed
    GATED = "GATED"  # GATed
    INDEX = "INDEX"  # INDex
    JUMP_MODE = "JUMP_MODE"  # jump_mode
    JUMP_TARGET = "JUMP_TARGET"  # jump_target
    LAST = "LAST"
    NEGATIVE = "NEGATIVE"  # NEGative
    NEXT = "NEXT"
    OFF = "OFF"
    POSITIVE = "POSITIVE"  # POSitive
    SEQUENCE = "SEQUENCE"  # SEQuence
    TRIGGERED = "TRIGGERED"  # TRIGgered
    VARIABLE = "VARIABLE"  # VARiable


#  pylint: disable=too-many-instance-attributes,too-many-public-methods
class AWG7KCCommands:
    """The AWG7KC commands.

    This provides access to all the commands for the AWG7KC device. See the documentation of each
    property for more usage information.

    Properties:
        - ``.abort``: The ``ABORt`` command.
        - ``.awgcontrol``: The ``AWGControl`` command tree.
        - ``.cal``: The ``*CAL`` command.
        - ``.calibration``: The ``CALibration`` command tree.
        - ``.cls``: The ``*CLS`` command.
        - ``.diagnostic``: The ``DIAGnostic`` command tree.
        - ``.display``: The ``DISPlay`` command tree.
        - ``.ese``: The ``*ESE`` command.
        - ``.esr``: The ``*ESR`` command.
        - ``.event``: The ``EVENt`` command tree.
        - ``.idn``: The ``*IDN`` command.
        - ``.instrument``: The ``INSTrument`` command tree.
        - ``.mmemory``: The ``MMEMory`` command tree.
        - ``.opc``: The ``*OPC`` command.
        - ``.opt``: The ``*OPT`` command.
        - ``.output``: The ``OUTPut[n]`` command tree.
        - ``.rst``: The ``*RST`` command.
        - ``.sequence``: The ``SEQuence`` command tree.
        - ``.slist``: The ``SLISt`` command tree.
        - ``.source``: The ``SOURce[n]`` command tree.
        - ``.sre``: The ``*SRE`` command.
        - ``.status``: The ``STATus`` command tree.
        - ``.stb``: The ``*STB`` command.
        - ``.system``: The ``SYSTem`` command tree.
        - ``.trg``: The ``*TRG`` command.
        - ``.trigger``: The ``TRIGger`` command tree.
        - ``.tst``: The ``*TST`` command.
        - ``.wai``: The ``*WAI`` command.
        - ``.wlist``: The ``WLISt`` command tree.
    """

    def __init__(self, device: Optional[PIDevice] = None) -> None:
        self._abort = Abort(device)
        self._awgcontrol = Awgcontrol(device)
        self._cal = Cal(device)
        self._calibration = Calibration(device)
        self._cls = Cls(device)
        self._diagnostic = Diagnostic(device)
        self._display = Display(device)
        self._ese = Ese(device)
        self._esr = Esr(device)
        self._event = Event(device)
        self._idn = Idn(device)
        self._instrument = Instrument(device)
        self._mmemory = Mmemory(device)
        self._opc = Opc(device)
        self._opt = Opt(device)
        self._output: Dict[int, OutputItem] = DefaultDictPassKeyToFactory(
            lambda x: OutputItem(device, f"OUTPut{x}")
        )
        self._rst = Rst(device)
        self._sequence = Sequence(device)
        self._slist = Slist(device)
        self._source: Dict[int, SourceItem] = DefaultDictPassKeyToFactory(
            lambda x: SourceItem(device, f"SOURce{x}")
        )
        self._sre = Sre(device)
        self._status = Status(device)
        self._stb = Stb(device)
        self._system = System(device)
        self._trg = Trg(device)
        self._trigger = Trigger(device)
        self._tst = Tst(device)
        self._wai = Wai(device)
        self._wlist = Wlist(device)

    @property
    def abort(self) -> Abort:
        """Return the ``ABORt`` command.

        **Description:**
            - Initializes all the current trigger system parameters and resets all trigger
              sequences.

        **Usage:**
            - Using the ``.write()`` method will send the ``ABORt`` command.

        **SCPI Syntax:**

        ::

            - ABORt
        """
        return self._abort

    @property
    def awgcontrol(self) -> Awgcontrol:
        """Return the ``AWGControl`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.application``: The ``AWGControl:APPLication`` command tree.
            - ``.clock``: The ``AWGControl:CLOCk`` command tree.
            - ``.compile``: The ``AWGControl:COMPile`` command.
            - ``.configure``: The ``AWGControl:CONFigure`` command tree.
            - ``.dc``: The ``AWGControl:DC[n]`` command tree.
            - ``.doutput``: The ``AWGControl:DOUTput[n]`` command tree.
            - ``.enhanced``: The ``AWGControl:ENHanced`` command tree.
            - ``.event``: The ``AWGControl:EVENt`` command tree.
            - ``.interleave``: The ``AWGControl:INTerleave`` command tree.
            - ``.rmode``: The ``AWGControl:RMODe`` command.
            - ``.rrate``: The ``AWGControl:RRATe`` command.
            - ``.rstate``: The ``AWGControl:RSTate`` command.
            - ``.run``: The ``AWGControl:RUN`` command tree.
            - ``.sequencer``: The ``AWGControl:SEQuencer`` command tree.
            - ``.sname``: The ``AWGControl:SNAMe`` command.
            - ``.srestore``: The ``AWGControl:SREStore`` command.
            - ``.ssave``: The ``AWGControl:SSAVe`` command.
            - ``.stop``: The ``AWGControl:STOP`` command tree.
        """
        return self._awgcontrol

    @property
    def cal(self) -> Cal:
        """Return the ``*CAL`` command.

        **Description:**
            - This query-only command starts signal path calibration (SPC) and returns the status
              upon completion.

        **Usage:**
            - Using the ``.query()`` method will send the ``*CAL?`` query.
            - Using the ``.verify(value)`` method will send the ``*CAL?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - *CAL?
        """
        return self._cal

    @property
    def calibration(self) -> Calibration:
        """Return the ``CALibration`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``CALibration?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibration?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.all``: The ``CALibration:ALL`` command.
        """
        return self._calibration

    @property
    def cls(self) -> Cls:
        """Return the ``*CLS`` command.

        **Description:**
            - This command (no query form) clears the following: Event Queue Standard Event Status
              Register Status Byte Register (except the MAV bit) If the ``*CLS`` command immediately
              follows an <EOI>, the Output Queue and MAV bit (Status Byte Register bit 4) are also
              cleared. MAV indicates that information is in the output queue. The device clear (DCL)
              control message will clear the output queue and thus MAV. ``*CLS`` does not clear the
              output queue or MAV. ``*CLS`` can suppress a Service Request that is to be generated
              by an ``*OPC``. This will happen if a single sequence acquisition operation is still
              being processed when the ``*CLS`` command is executed.

        **Usage:**
            - Using the ``.write()`` method will send the ``*CLS`` command.

        **SCPI Syntax:**

        ::

            - *CLS
        """
        return self._cls

    @property
    def diagnostic(self) -> Diagnostic:
        """Return the ``DIAGnostic`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``DIAGnostic?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAGnostic?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.data``: The ``DIAGnostic:DATA`` command.
            - ``.select``: The ``DIAGnostic:SELect`` command.
            - ``.immediate``: The ``DIAGnostic:IMMediate`` command.
        """
        return self._diagnostic

    @property
    def display(self) -> Display:
        """Return the ``DISPlay`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``DISPlay?`` query.
            - Using the ``.verify(value)`` method will send the ``DISPlay?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.window1``: The ``DISPlay:WINDow1`` command tree.
            - ``.window2``: The ``DISPlay:WINDow2`` command tree.
        """
        return self._display

    @property
    def ese(self) -> Ese:
        """Return the ``*ESE`` command.

        **Description:**
            - This command sets and queries the bits in the Event Status Enable Register (ESER). The
              ESER prevents events from being reported to the Status Byte Register (STB). For a more
              detailed discussion of the use of these registers, see Registers.

        **Usage:**
            - Using the ``.query()`` method will send the ``*ESE?`` query.
            - Using the ``.verify(value)`` method will send the ``*ESE?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``*ESE value`` command.

        **SCPI Syntax:**

        ::

            - *ESE <NR1>
            - *ESE?

        **Info:**
            - ``<NR1>`` specifies the binary bits of the ESER according to this value, which ranges
              from 0 through 255.
        """
        return self._ese

    @property
    def esr(self) -> Esr:
        """Return the ``*ESR`` command.

        **Description:**
            - This query-only command returns the contents of the Standard Event Status Register
              (SESR). ``*ESR?`` also clears the SESR (since reading the SESR clears it). For a more
              detailed discussion of the use of these registers, see Registers.

        **Usage:**
            - Using the ``.query()`` method will send the ``*ESR?`` query.
            - Using the ``.verify(value)`` method will send the ``*ESR?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - *ESR?
        """
        return self._esr

    @property
    def event(self) -> Event:
        """Return the ``EVENt`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``EVENt?`` query.
            - Using the ``.verify(value)`` method will send the ``EVENt?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.impedance``: The ``EVENt:IMPedance`` command.
            - ``.jtiming``: The ``EVENt:JTIMing`` command.
            - ``.level``: The ``EVENt:LEVel`` command.
            - ``.polarity``: The ``EVENt:POLarity`` command.
            - ``.immediate``: The ``EVENt:IMMediate`` command.
        """
        return self._event

    @property
    def idn(self) -> Idn:
        """Return the ``*IDN`` command.

        **Description:**
            - This query-only command returns the instrument identification code.

        **Usage:**
            - Using the ``.query()`` method will send the ``*IDN?`` query.
            - Using the ``.verify(value)`` method will send the ``*IDN?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - *IDN?
        """
        return self._idn

    @property
    def instrument(self) -> Instrument:
        """Return the ``INSTrument`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``INSTrument?`` query.
            - Using the ``.verify(value)`` method will send the ``INSTrument?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.couple``: The ``INSTrument:COUPle`` command tree.
        """
        return self._instrument

    @property
    def mmemory(self) -> Mmemory:
        """Return the ``MMEMory`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``MMEMory?`` query.
            - Using the ``.verify(value)`` method will send the ``MMEMory?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.catalog``: The ``MMEMory:CATalog`` command.
            - ``.cdirectory``: The ``MMEMory:CDIRectory`` command.
            - ``.data``: The ``MMEMory:DATA`` command.
            - ``.delete``: The ``MMEMory:DELete`` command.
            - ``.export``: The ``MMEMory:EXPort`` command.
            - ``.import``: The ``MMEMory:IMPort`` command.
            - ``.mdirectory``: The ``MMEMory:MDIRectory`` command.
            - ``.msis``: The ``MMEMory:MSIS`` command.
        """
        return self._mmemory

    @property
    def opc(self) -> Opc:
        """Return the ``*OPC`` command.

        **Description:**
            - This command generates the operation complete message in the Standard Event Status
              Register (SESR) when all pending commands that generate an OPC message are complete.
              The ``*OPC?`` query places the ASCII character '1' into the output queue when all such
              OPC commands are complete. The ``*OPC?`` response is not available to read until all
              pending operations finish. For a complete discussion of the use of these registers and
              the output queue, see Registers and Queues. The ``*OPC`` command allows you to
              synchronize the operation of the instrument with your application program. For more
              information, see Synchronization Methods. Refer to the Oscilloscope operations that
              can generate OPC table for a list of commands that generate an OPC message.

        **Usage:**
            - Using the ``.query()`` method will send the ``*OPC?`` query.
            - Using the ``.verify(value)`` method will send the ``*OPC?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write()`` method will send the ``*OPC`` command.

        **SCPI Syntax:**

        ::

            - *OPC
            - *OPC?
        """
        return self._opc

    @property
    def opt(self) -> Opt:
        """Return the ``*OPT`` command.

        **Description:**
            - This query-only command returns a comma separated list of installed options as an
              arbitrary ASCII string (no quotes) of the form:
              ``<optionCode>:<optionDescription>``,``<optionCode>:<optionDescription>``... The last
              section of each entry (the text following the last hyphen) indicates the license type.
              If no options are found, NONE is returned.

        **Usage:**
            - Using the ``.query()`` method will send the ``*OPT?`` query.
            - Using the ``.verify(value)`` method will send the ``*OPT?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - *OPT?
        """
        return self._opt

    @property
    def output(self) -> Dict[int, OutputItem]:
        """Return the ``OUTPut[n]`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.filter``: The ``OUTPut[n]:FILTer`` command tree.
            - ``.state``: The ``OUTPut[n]:STATe`` command.
        """
        return self._output

    @property
    def rst(self) -> Rst:
        """Return the ``*RST`` command.

        **Description:**
            - This command (no query form) resets the instrument to the factory default settings.
              This command does the following: Recalls the default instrument setup. Clears the
              current ``*DDT`` command. Disables aliases (``:ALIAS:STATE 0``). Disables the user
              password (for the ``*PUD`` command). The ``*RST`` command does not change the
              following: The current working directory ( ``:FILESystem:CWD`` command). The state of
              command headers ( ``:HEADer`` command). The state of keyword and enumeration verbosity
              ( ``:VERBose`` command). The Power-on Status Clear Flag ( ``*PSC`` command). The Event
              Status Enable Register ( ``*ESE`` command). The Service Request Enable Register (
              ``*SRE`` command). The Device Event Status Enable Register ( DESE command). The user
              password for protected user data ( ``:PASSWord`` command). The content of protected
              user data ( ``*PUD`` command). The enabled state of the socket server (
              ``:SOCKETServer:ENAble`` command). The socket server port number (
              ``:SOCKETServer:PORT`` command). The socket server protocol (
              ``:SOCKETServer:PROTOCol`` command). The USBTMC port configuration (
              ``:USBDevice:CONFigure`` command). The destination reference waveform or file path for
              the ``:CURVe`` command ( ``:DATa:DESTination`` command). The source waveform for the
              ``:CURVe?`` or ``:WAVFrm?`` queries ( ``:DATa:SOUrce`` command). The waveform data
              encoding for the ``:CURVe`` command or query or the ``:WAVFrm?`` query (
              ``:DATa:ENCdg`` command). The starting point for ``:CURVe?`` queries ( ``:DATa:STARt``
              command). The ending point for ``:CURVe?`` queries ( ``:DATa:STOP`` command). All
              settings associated the ``:WFMInpre`` commands. All user settable settings associated
              with the WFMOutpre commands. ``*RST`` only resets the programmable interface settings,
              it does not change the user interface settings.

        **Usage:**
            - Using the ``.write()`` method will send the ``*RST`` command.

        **SCPI Syntax:**

        ::

            - *RST
        """
        return self._rst

    @property
    def sequence(self) -> Sequence:
        """Return the ``SEQuence`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SEQuence?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.element``: The ``SEQuence:ELEMent[n]`` command tree.
            - ``.jump``: The ``SEQuence:JUMP`` command tree.
            - ``.length``: The ``SEQuence:LENGth`` command.
        """
        return self._sequence

    @property
    def slist(self) -> Slist:
        """Return the ``SLISt`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.name``: The ``SLISt:NAME`` command.
            - ``.size``: The ``SLISt:SIZE`` command.
            - ``.subsequence``: The ``SLISt:SUBSequence`` command tree.
        """
        return self._slist

    @property
    def source(self) -> Dict[int, SourceItem]:
        """Return the ``SOURce[n]`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SOURce[n]?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.frequency``: The ``SOURce[n]:FREQuency`` command tree.
            - ``.roscillator``: The ``SOURce[n]:ROSCillator`` command tree.
            - ``.combine``: The ``SOURce[n]:COMBine`` command tree.
            - ``.dac``: The ``SOURce[n]:DAC`` command tree.
            - ``.delay``: The ``SOURce[n]:DELay`` command tree.
            - ``.digital``: The ``SOURce[n]:DIGital`` command tree.
            - ``.function``: The ``SOURce[n]:FUNCtion`` command tree.
            - ``.marker1``: The ``SOURce[n]:MARKer1`` command tree.
            - ``.marker2``: The ``SOURce[n]:MARKer2`` command tree.
            - ``.pdelay``: The ``SOURce[n]:PDELay`` command tree.
            - ``.phase``: The ``SOURce[n]:PHASe`` command tree.
            - ``.skew``: The ``SOURce[n]:SKEW`` command.
            - ``.voltage``: The ``SOURce[n]:VOLTage`` command tree.
            - ``.waveform``: The ``SOURce[n]:WAVeform`` command.
        """
        return self._source

    @property
    def sre(self) -> Sre:
        """Return the ``*SRE`` command.

        **Description:**
            - The ``*SRE`` (Service Request Enable) command sets and queries the bits in the Service
              Request Enable Register. For more information, refer to Registers.

        **Usage:**
            - Using the ``.query()`` method will send the ``*SRE?`` query.
            - Using the ``.verify(value)`` method will send the ``*SRE?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``*SRE value`` command.

        **SCPI Syntax:**

        ::

            - *SRE <NR1>
            - *SRE?

        **Info:**
            - ``<NR1>`` is a value in the range from 0 through 255. The binary bits of the SRER are
              set according to this value. Using an out-of-range value causes an execution error.
              The power-on default for SRER is 0 if ``*PSC`` is 1. If ``*PSC`` is 0, the SRER
              maintains the previous power cycle value through the current power cycle.
        """
        return self._sre

    @property
    def status(self) -> Status:
        """Return the ``STATus`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``STATus?`` query.
            - Using the ``.verify(value)`` method will send the ``STATus?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.operation``: The ``STATus:OPERation`` command tree.
            - ``.preset``: The ``STATus:PRESet`` command.
            - ``.questionable``: The ``STATus:QUEStionable`` command tree.
        """
        return self._status

    @property
    def stb(self) -> Stb:
        """Return the ``*STB`` command.

        **Description:**
            - The ``*STB?`` (Read Status Byte) query returns the contents of the Status Byte
              Register (SBR) using the Master Summary Status (MSS) bit. For more information, refer
              to Registers.

        **Usage:**
            - Using the ``.query()`` method will send the ``*STB?`` query.
            - Using the ``.verify(value)`` method will send the ``*STB?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - *STB?
        """
        return self._stb

    @property
    def system(self) -> System:
        """Return the ``SYSTem`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SYSTem?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.date``: The ``SYSTem:DATE`` command.
            - ``.error``: The ``SYSTem:ERRor`` command tree.
            - ``.klock``: The ``SYSTem:KLOCk`` command.
            - ``.time``: The ``SYSTem:TIME`` command.
            - ``.version``: The ``SYSTem:VERSion`` command.
        """
        return self._system

    @property
    def trg(self) -> Trg:
        """Return the ``*TRG`` command.

        **Description:**
            - Performs a group execute trigger on commands defined by ``*DDT``.

        **Usage:**
            - Using the ``.write()`` method will send the ``*TRG`` command.

        **SCPI Syntax:**

        ::

            - *TRG
        """
        return self._trg

    @property
    def trigger(self) -> Trigger:
        """Return the ``TRIGger`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``TRIGger?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.sequence``: The ``TRIGger:SEQuence`` command tree.
        """
        return self._trigger

    @property
    def tst(self) -> Tst:
        """Return the ``*TST`` command.

        **Description:**
            - Tests (self-test) the interface and returns a 0.

        **Usage:**
            - Using the ``.query()`` method will send the ``*TST?`` query.
            - Using the ``.verify(value)`` method will send the ``*TST?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - *TST?
        """
        return self._tst

    @property
    def wai(self) -> Wai:
        """Return the ``*WAI`` command.

        **Description:**
            - The ``*WAI`` (Wait) command (no query form) prevents the instrument from executing
              further commands or queries until all pending commands that generate an OPC message
              are complete. This command allows you to synchronize the operation of the instrument
              with your application program. For more information, refer to Synchronization Methods.

        **Usage:**
            - Using the ``.write()`` method will send the ``*WAI`` command.

        **SCPI Syntax:**

        ::

            - *WAI
        """
        return self._wai

    @property
    def wlist(self) -> Wlist:
        """Return the ``WLISt`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``WLISt?`` query.
            - Using the ``.verify(value)`` method will send the ``WLISt?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.name``: The ``WLISt:NAME`` command.
            - ``.size``: The ``WLISt:SIZE`` command.
            - ``.waveform``: The ``WLISt:WAVeform`` command tree.
        """
        return self._wlist


class AWG7KCMixin:
    """A mixin that provides access to the AWG7KC commands and constants.

    Properties:
        - ``.command_argument_constants``: The AWG7KC command argument constants.
        - ``.commands``: The AWG7KC commands.
    """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        device = self if isinstance(self, PIDevice) else None
        self._command_argument_constants = AWG7KCCommandConstants()
        self._commands = AWG7KCCommands(device)

    @property
    def command_argument_constants(self) -> AWG7KCCommandConstants:
        """Return the AWG7KC command argument constants.

        This provides access to all the string constants which can be used as arguments for AWG7KC
        commands.
        """
        return self._command_argument_constants

    @property
    def commands(self) -> AWG7KCCommands:
        """Return the AWG7KC commands.

        This provides access to all the commands for the AWG7KC device. See the documentation of
        each sub-property for more usage information.

        Sub-properties:
            - ``.abort``: The ``ABORt`` command.
            - ``.awgcontrol``: The ``AWGControl`` command tree.
            - ``.cal``: The ``*CAL`` command.
            - ``.calibration``: The ``CALibration`` command tree.
            - ``.cls``: The ``*CLS`` command.
            - ``.diagnostic``: The ``DIAGnostic`` command tree.
            - ``.display``: The ``DISPlay`` command tree.
            - ``.ese``: The ``*ESE`` command.
            - ``.esr``: The ``*ESR`` command.
            - ``.event``: The ``EVENt`` command tree.
            - ``.idn``: The ``*IDN`` command.
            - ``.instrument``: The ``INSTrument`` command tree.
            - ``.mmemory``: The ``MMEMory`` command tree.
            - ``.opc``: The ``*OPC`` command.
            - ``.opt``: The ``*OPT`` command.
            - ``.output``: The ``OUTPut[n]`` command tree.
            - ``.rst``: The ``*RST`` command.
            - ``.sequence``: The ``SEQuence`` command tree.
            - ``.slist``: The ``SLISt`` command tree.
            - ``.source``: The ``SOURce[n]`` command tree.
            - ``.sre``: The ``*SRE`` command.
            - ``.status``: The ``STATus`` command tree.
            - ``.stb``: The ``*STB`` command.
            - ``.system``: The ``SYSTem`` command tree.
            - ``.trg``: The ``*TRG`` command.
            - ``.trigger``: The ``TRIGger`` command tree.
            - ``.tst``: The ``*TST`` command.
            - ``.wai``: The ``*WAI`` command.
            - ``.wlist``: The ``WLISt`` command tree.
        """
        return self._commands



"""The delete commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - DELEte:SETUp {ALL|<NR1>}
    - DELEte:WAVEform {ALL|REF<x>}
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class DeleteWaveform(SCPICmdWrite):
    """The ``DELEte:WAVEform`` command.

    **Description:**
        - This command (no query form) deletes one or all stored reference waveforms from memory.
          This command is equivalent to selecting Delete from the File menu, and then choosing the
          reference waveform you want to delete; choosing All Refs deletes all of the reference
          waveforms.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``DELEte:WAVEform value`` command.

    **SCPI Syntax:**

    ::

        - DELEte:WAVEform {ALL|REF<x>}

    **Info:**
        - ``ALL`` specifies to delete all the stored reference waveforms.
        - ``REF<x>`` specifies to delete one of the reference memory locations. Reference memory
          location values range from 1 through 4.
    """


class DeleteSetup(SCPICmdWrite):
    """The ``DELEte:SETUp`` command.

    **Description:**
        - This command (no query form) changes the setup to reference the factory setup instead of
          the specific user setup slot. The content of the setup slot is unchanged, but the data
          will no longer be accessible to you. This command is equivalent to selecting Delete from
          the File menu, and then clicking the specific setup you want to delete (user setups are
          shown as Setup-User and are ordered in the list from 1 through 10, if defined) or All
          Setups.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``DELEte:SETUp value`` command.

    **SCPI Syntax:**

    ::

        - DELEte:SETUp {ALL|<NR1>}

    **Info:**
        - ``ALL`` deletes all the stored setups.
        - ``<NR1>`` specifies a setup storage location to delete. Setup storage location values
          range from 1 through 10; using an out-of-range value causes an error.
    """


class Delete(SCPICmdRead):
    """The ``DELEte`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``DELEte?`` query.
        - Using the ``.verify(value)`` method will send the ``DELEte?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.setup``: The ``DELEte:SETUp`` command.
        - ``.waveform``: The ``DELEte:WAVEform`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "DELEte") -> None:
        super().__init__(device, cmd_syntax)
        self._setup = DeleteSetup(device, f"{self._cmd_syntax}:SETUp")
        self._waveform = DeleteWaveform(device, f"{self._cmd_syntax}:WAVEform")

    @property
    def setup(self) -> DeleteSetup:
        """Return the ``DELEte:SETUp`` command.

        **Description:**
            - This command (no query form) changes the setup to reference the factory setup instead
              of the specific user setup slot. The content of the setup slot is unchanged, but the
              data will no longer be accessible to you. This command is equivalent to selecting
              Delete from the File menu, and then clicking the specific setup you want to delete
              (user setups are shown as Setup-User and are ordered in the list from 1 through 10, if
              defined) or All Setups.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``DELEte:SETUp value`` command.

        **SCPI Syntax:**

        ::

            - DELEte:SETUp {ALL|<NR1>}

        **Info:**
            - ``ALL`` deletes all the stored setups.
            - ``<NR1>`` specifies a setup storage location to delete. Setup storage location values
              range from 1 through 10; using an out-of-range value causes an error.
        """
        return self._setup

    @property
    def waveform(self) -> DeleteWaveform:
        """Return the ``DELEte:WAVEform`` command.

        **Description:**
            - This command (no query form) deletes one or all stored reference waveforms from
              memory. This command is equivalent to selecting Delete from the File menu, and then
              choosing the reference waveform you want to delete; choosing All Refs deletes all of
              the reference waveforms.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``DELEte:WAVEform value`` command.

        **SCPI Syntax:**

        ::

            - DELEte:WAVEform {ALL|REF<x>}

        **Info:**
            - ``ALL`` specifies to delete all the stored reference waveforms.
            - ``REF<x>`` specifies to delete one of the reference memory locations. Reference memory
              location values range from 1 through 4.
        """
        return self._waveform



"""The message commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2K, MSO2KB, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - MESSage
    - MESSage:BOX <X1>,<Y1>[,<X2>,<Y2>]
    - MESSage:BOX?
    - MESSage:CLEAR
    - MESSage:SHOW <QString>
    - MESSage:SHOW?
    - MESSage:STATE {OFF|ON|<NR1>}
    - MESSage:STATE?
    - MESSage?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class MessageState(SCPICmdWrite, SCPICmdRead):
    """The ``MESSage:STATE`` command.

    **Description:**
        - Controls the display of the message box.

    **Usage:**
        - Using the ``.query()`` method will send the ``MESSage:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``MESSage:STATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MESSage:STATE value`` command.

    **SCPI Syntax:**

    ::

        - MESSage:STATE {OFF|ON|<NR1>}
        - MESSage:STATE?

    **Info:**
        - ``OFF or <NR1> = 0`` removes the message window from the screen.
        - ``ON or <NR1> ≠ 0`` displays the message window and its contents on the screen.
    """


class MessageShow(SCPICmdWrite, SCPICmdRead):
    """The ``MESSage:SHOW`` command.

    **Description:**
        - This command specifies the contents of the message box. ``MESSage:SHOW`` <Qstring> defines
          the content of the message box. Change in string length causes automatic resize of the
          message box to fit the text. The box may be resized using the ``MESSAGE:BOX`` command. The
          ``MESSAGE:STATE`` command is used to turn on and off the message box display.

    **Usage:**
        - Using the ``.query()`` method will send the ``MESSage:SHOW?`` query.
        - Using the ``.verify(value)`` method will send the ``MESSage:SHOW?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MESSage:SHOW value`` command.

    **SCPI Syntax:**

    ::

        - MESSage:SHOW <QString>
        - MESSage:SHOW?

    **Info:**
        - ``<QString>`` is the message and can include any of the characters shown in the Character
          Set.
    """

    _WRAP_ARG_WITH_QUOTES = True


class MessageClear(SCPICmdWriteNoArguments):
    """The ``MESSage:CLEAR`` command.

    **Description:**
        - Clears the contents of the message box.

    **Usage:**
        - Using the ``.write()`` method will send the ``MESSage:CLEAR`` command.

    **SCPI Syntax:**

    ::

        - MESSage:CLEAR
    """


class MessageBox(SCPICmdWrite, SCPICmdRead):
    """The ``MESSage:BOX`` command.

    **Description:**
        - This command specifies the co-ordinates of the message box. This command does not display
          the message unless ``MESSage:STATE`` is on. X1 and Y1 are the screen coordinates of the
          top left corner of the message box. X2 and Y2 are the screen coordinates of the bottom
          right corner of the message box. All four coordinates are returned by the query. Changing
          the text in the message box, using the ``MESSAGE:SHOW`` command, automatically resizes the
          message box. If you want a custom message box size, send the ``MESSAGE:BOX`` command after
          changing the text using the ``MESSAGE:SHOW`` command. Message box settings and data are
          saved and restored in saved setups.

    **Usage:**
        - Using the ``.query()`` method will send the ``MESSage:BOX?`` query.
        - Using the ``.verify(value)`` method will send the ``MESSage:BOX?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MESSage:BOX value`` command.

    **SCPI Syntax:**

    ::

        - MESSage:BOX <X1>,<Y1>[,<X2>,<Y2>]
        - MESSage:BOX?

    **Info:**
        - ``<X1>`` and <X2> = 0 to 1023, and are pixel positions along the horizontal axis. <X1>
          defines the left and <X2> defines the right side of the window.
        - ``<Y1>`` and <Y2> = 0 to 767, and are pixel positions along the vertical axis. <Y1>
          defines the top and <Y2> defines the bottom of the window. The reserved height of all
          characters is 16 pixels so the window must be at least that high to fully display
          characters. <X2> and <Y2> are optional because the ``MESSAGE:SHOW`` command automatically
          sizes the box to fit the message. All four values are returned in a query.
    """


class Message(SCPICmdWriteNoArguments, SCPICmdRead):
    """The ``MESSage`` command.

    **Description:**
        - This command sets or queries message box (screen annotation) parameters.

    **Usage:**
        - Using the ``.query()`` method will send the ``MESSage?`` query.
        - Using the ``.verify(value)`` method will send the ``MESSage?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write()`` method will send the ``MESSage`` command.

    **SCPI Syntax:**

    ::

        - MESSage
        - MESSage?

    Properties:
        - ``.box``: The ``MESSage:BOX`` command.
        - ``.clear``: The ``MESSage:CLEAR`` command.
        - ``.show``: The ``MESSage:SHOW`` command.
        - ``.state``: The ``MESSage:STATE`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "MESSage") -> None:
        super().__init__(device, cmd_syntax)
        self._box = MessageBox(device, f"{self._cmd_syntax}:BOX")
        self._clear = MessageClear(device, f"{self._cmd_syntax}:CLEAR")
        self._show = MessageShow(device, f"{self._cmd_syntax}:SHOW")
        self._state = MessageState(device, f"{self._cmd_syntax}:STATE")

    @property
    def box(self) -> MessageBox:
        """Return the ``MESSage:BOX`` command.

        **Description:**
            - This command specifies the co-ordinates of the message box. This command does not
              display the message unless ``MESSage:STATE`` is on. X1 and Y1 are the screen
              coordinates of the top left corner of the message box. X2 and Y2 are the screen
              coordinates of the bottom right corner of the message box. All four coordinates are
              returned by the query. Changing the text in the message box, using the
              ``MESSAGE:SHOW`` command, automatically resizes the message box. If you want a custom
              message box size, send the ``MESSAGE:BOX`` command after changing the text using the
              ``MESSAGE:SHOW`` command. Message box settings and data are saved and restored in
              saved setups.

        **Usage:**
            - Using the ``.query()`` method will send the ``MESSage:BOX?`` query.
            - Using the ``.verify(value)`` method will send the ``MESSage:BOX?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MESSage:BOX value`` command.

        **SCPI Syntax:**

        ::

            - MESSage:BOX <X1>,<Y1>[,<X2>,<Y2>]
            - MESSage:BOX?

        **Info:**
            - ``<X1>`` and <X2> = 0 to 1023, and are pixel positions along the horizontal axis. <X1>
              defines the left and <X2> defines the right side of the window.
            - ``<Y1>`` and <Y2> = 0 to 767, and are pixel positions along the vertical axis. <Y1>
              defines the top and <Y2> defines the bottom of the window. The reserved height of all
              characters is 16 pixels so the window must be at least that high to fully display
              characters. <X2> and <Y2> are optional because the ``MESSAGE:SHOW`` command
              automatically sizes the box to fit the message. All four values are returned in a
              query.
        """
        return self._box

    @property
    def clear(self) -> MessageClear:
        """Return the ``MESSage:CLEAR`` command.

        **Description:**
            - Clears the contents of the message box.

        **Usage:**
            - Using the ``.write()`` method will send the ``MESSage:CLEAR`` command.

        **SCPI Syntax:**

        ::

            - MESSage:CLEAR
        """
        return self._clear

    @property
    def show(self) -> MessageShow:
        """Return the ``MESSage:SHOW`` command.

        **Description:**
            - This command specifies the contents of the message box. ``MESSage:SHOW`` <Qstring>
              defines the content of the message box. Change in string length causes automatic
              resize of the message box to fit the text. The box may be resized using the
              ``MESSAGE:BOX`` command. The ``MESSAGE:STATE`` command is used to turn on and off the
              message box display.

        **Usage:**
            - Using the ``.query()`` method will send the ``MESSage:SHOW?`` query.
            - Using the ``.verify(value)`` method will send the ``MESSage:SHOW?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MESSage:SHOW value`` command.

        **SCPI Syntax:**

        ::

            - MESSage:SHOW <QString>
            - MESSage:SHOW?

        **Info:**
            - ``<QString>`` is the message and can include any of the characters shown in the
              Character Set.
        """
        return self._show

    @property
    def state(self) -> MessageState:
        """Return the ``MESSage:STATE`` command.

        **Description:**
            - Controls the display of the message box.

        **Usage:**
            - Using the ``.query()`` method will send the ``MESSage:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``MESSage:STATE?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MESSage:STATE value`` command.

        **SCPI Syntax:**

        ::

            - MESSage:STATE {OFF|ON|<NR1>}
            - MESSage:STATE?

        **Info:**
            - ``OFF or <NR1> = 0`` removes the message window from the screen.
            - ``ON or <NR1> ≠ 0`` displays the message window and its contents on the screen.
        """
        return self._state



"""The email commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - EMAIL:SETUp:FROMADDRess <QString>
    - EMAIL:SETUp:FROMADDRess?
    - EMAIL:SETUp:HOSTALIASNAMe <QString>
    - EMAIL:SETUp:HOSTALIASNAMe?
    - EMAIL:SETUp:SMTPLOGIn <QString>
    - EMAIL:SETUp:SMTPLOGIn?
    - EMAIL:SETUp:SMTPPASSWord <QString>
    - EMAIL:SETUp:SMTPPort <NR1>
    - EMAIL:SETUp:SMTPPort?
    - EMAIL:SETUp:SMTPServer <QString>
    - EMAIL:SETUp:SMTPServer?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class EmailSetupSmtpserver(SCPICmdWrite, SCPICmdRead):
    """The ``EMAIL:SETUp:SMTPServer`` command.

    **Description:**
        - Sets or returns the email SMTP server DNS name for the common server setup information
          that is shared between the Act on Event commands and the Hardcopy Email commands.

    **Usage:**
        - Using the ``.query()`` method will send the ``EMAIL:SETUp:SMTPServer?`` query.
        - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp:SMTPServer?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:SMTPServer value`` command.

    **SCPI Syntax:**

    ::

        - EMAIL:SETUp:SMTPServer <QString>
        - EMAIL:SETUp:SMTPServer?
    """

    _WRAP_ARG_WITH_QUOTES = True


class EmailSetupSmtpport(SCPICmdWrite, SCPICmdRead):
    """The ``EMAIL:SETUp:SMTPPort`` command.

    **Description:**
        - Sets or returns the email SMTP server port number for the common server setup information
          that is shared between the Act on Event commands and the Hardcopy Email commands. The
          default port number is 25.

    **Usage:**
        - Using the ``.query()`` method will send the ``EMAIL:SETUp:SMTPPort?`` query.
        - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp:SMTPPort?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:SMTPPort value`` command.

    **SCPI Syntax:**

    ::

        - EMAIL:SETUp:SMTPPort <NR1>
        - EMAIL:SETUp:SMTPPort?
    """


class EmailSetupSmtppassword(SCPICmdWrite):
    """The ``EMAIL:SETUp:SMTPPASSWord`` command.

    **Description:**
        - Sets the email SMTP server login password for the common server setup information that is
          shared between the Act on Event commands and the Hardcopy Email commands. For security
          reasons, no query form is provided.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:SMTPPASSWord value``
          command.

    **SCPI Syntax:**

    ::

        - EMAIL:SETUp:SMTPPASSWord <QString>
    """

    _WRAP_ARG_WITH_QUOTES = True


class EmailSetupSmtplogin(SCPICmdWrite, SCPICmdRead):
    """The ``EMAIL:SETUp:SMTPLOGIn`` command.

    **Description:**
        - Sets or returns the email SMTP server login ID for the common server setup information
          that is shared between the Act on Event commands and the Hardcopy Email commands.

    **Usage:**
        - Using the ``.query()`` method will send the ``EMAIL:SETUp:SMTPLOGIn?`` query.
        - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp:SMTPLOGIn?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:SMTPLOGIn value`` command.

    **SCPI Syntax:**

    ::

        - EMAIL:SETUp:SMTPLOGIn <QString>
        - EMAIL:SETUp:SMTPLOGIn?
    """

    _WRAP_ARG_WITH_QUOTES = True


class EmailSetupHostaliasname(SCPICmdWrite, SCPICmdRead):
    """The ``EMAIL:SETUp:HOSTALIASNAMe`` command.

    **Description:**
        - Sets (or queries) the email host alias name for the common server setup information that
          is shared between the Act on Event commands and the Hardcopy Email commands. If this is an
          empty string, the DNS name of the instrument is used. This string is included in the email
          message.

    **Usage:**
        - Using the ``.query()`` method will send the ``EMAIL:SETUp:HOSTALIASNAMe?`` query.
        - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp:HOSTALIASNAMe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:HOSTALIASNAMe value``
          command.

    **SCPI Syntax:**

    ::

        - EMAIL:SETUp:HOSTALIASNAMe <QString>
        - EMAIL:SETUp:HOSTALIASNAMe?
    """

    _WRAP_ARG_WITH_QUOTES = True


class EmailSetupFromaddress(SCPICmdWrite, SCPICmdRead):
    """The ``EMAIL:SETUp:FROMADDRess`` command.

    **Description:**
        - Sets (or queries) the sender's email address for the common server setup information that
          is shared between the Act on Event commands and the Hardcopy Email commands. Note: to set
          the email recipient address for Act on Event commands, use
          ``ACTONEVENT:ACTION:EMAIL:SETUP:TOADDRESS``.

    **Usage:**
        - Using the ``.query()`` method will send the ``EMAIL:SETUp:FROMADDRess?`` query.
        - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp:FROMADDRess?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:FROMADDRess value``
          command.

    **SCPI Syntax:**

    ::

        - EMAIL:SETUp:FROMADDRess <QString>
        - EMAIL:SETUp:FROMADDRess?
    """

    _WRAP_ARG_WITH_QUOTES = True


class EmailSetup(SCPICmdRead):
    """The ``EMAIL:SETUp`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``EMAIL:SETUp?`` query.
        - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.fromaddress``: The ``EMAIL:SETUp:FROMADDRess`` command.
        - ``.hostaliasname``: The ``EMAIL:SETUp:HOSTALIASNAMe`` command.
        - ``.smtplogin``: The ``EMAIL:SETUp:SMTPLOGIn`` command.
        - ``.smtppassword``: The ``EMAIL:SETUp:SMTPPASSWord`` command.
        - ``.smtpport``: The ``EMAIL:SETUp:SMTPPort`` command.
        - ``.smtpserver``: The ``EMAIL:SETUp:SMTPServer`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._fromaddress = EmailSetupFromaddress(device, f"{self._cmd_syntax}:FROMADDRess")
        self._hostaliasname = EmailSetupHostaliasname(device, f"{self._cmd_syntax}:HOSTALIASNAMe")
        self._smtplogin = EmailSetupSmtplogin(device, f"{self._cmd_syntax}:SMTPLOGIn")
        self._smtppassword = EmailSetupSmtppassword(device, f"{self._cmd_syntax}:SMTPPASSWord")
        self._smtpport = EmailSetupSmtpport(device, f"{self._cmd_syntax}:SMTPPort")
        self._smtpserver = EmailSetupSmtpserver(device, f"{self._cmd_syntax}:SMTPServer")

    @property
    def fromaddress(self) -> EmailSetupFromaddress:
        """Return the ``EMAIL:SETUp:FROMADDRess`` command.

        **Description:**
            - Sets (or queries) the sender's email address for the common server setup information
              that is shared between the Act on Event commands and the Hardcopy Email commands.
              Note: to set the email recipient address for Act on Event commands, use
              ``ACTONEVENT:ACTION:EMAIL:SETUP:TOADDRESS``.

        **Usage:**
            - Using the ``.query()`` method will send the ``EMAIL:SETUp:FROMADDRess?`` query.
            - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp:FROMADDRess?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:FROMADDRess value``
              command.

        **SCPI Syntax:**

        ::

            - EMAIL:SETUp:FROMADDRess <QString>
            - EMAIL:SETUp:FROMADDRess?
        """
        return self._fromaddress

    @property
    def hostaliasname(self) -> EmailSetupHostaliasname:
        """Return the ``EMAIL:SETUp:HOSTALIASNAMe`` command.

        **Description:**
            - Sets (or queries) the email host alias name for the common server setup information
              that is shared between the Act on Event commands and the Hardcopy Email commands. If
              this is an empty string, the DNS name of the instrument is used. This string is
              included in the email message.

        **Usage:**
            - Using the ``.query()`` method will send the ``EMAIL:SETUp:HOSTALIASNAMe?`` query.
            - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp:HOSTALIASNAMe?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:HOSTALIASNAMe value``
              command.

        **SCPI Syntax:**

        ::

            - EMAIL:SETUp:HOSTALIASNAMe <QString>
            - EMAIL:SETUp:HOSTALIASNAMe?
        """
        return self._hostaliasname

    @property
    def smtplogin(self) -> EmailSetupSmtplogin:
        """Return the ``EMAIL:SETUp:SMTPLOGIn`` command.

        **Description:**
            - Sets or returns the email SMTP server login ID for the common server setup information
              that is shared between the Act on Event commands and the Hardcopy Email commands.

        **Usage:**
            - Using the ``.query()`` method will send the ``EMAIL:SETUp:SMTPLOGIn?`` query.
            - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp:SMTPLOGIn?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:SMTPLOGIn value``
              command.

        **SCPI Syntax:**

        ::

            - EMAIL:SETUp:SMTPLOGIn <QString>
            - EMAIL:SETUp:SMTPLOGIn?
        """
        return self._smtplogin

    @property
    def smtppassword(self) -> EmailSetupSmtppassword:
        """Return the ``EMAIL:SETUp:SMTPPASSWord`` command.

        **Description:**
            - Sets the email SMTP server login password for the common server setup information that
              is shared between the Act on Event commands and the Hardcopy Email commands. For
              security reasons, no query form is provided.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:SMTPPASSWord value``
              command.

        **SCPI Syntax:**

        ::

            - EMAIL:SETUp:SMTPPASSWord <QString>
        """
        return self._smtppassword

    @property
    def smtpport(self) -> EmailSetupSmtpport:
        """Return the ``EMAIL:SETUp:SMTPPort`` command.

        **Description:**
            - Sets or returns the email SMTP server port number for the common server setup
              information that is shared between the Act on Event commands and the Hardcopy Email
              commands. The default port number is 25.

        **Usage:**
            - Using the ``.query()`` method will send the ``EMAIL:SETUp:SMTPPort?`` query.
            - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp:SMTPPort?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:SMTPPort value``
              command.

        **SCPI Syntax:**

        ::

            - EMAIL:SETUp:SMTPPort <NR1>
            - EMAIL:SETUp:SMTPPort?
        """
        return self._smtpport

    @property
    def smtpserver(self) -> EmailSetupSmtpserver:
        """Return the ``EMAIL:SETUp:SMTPServer`` command.

        **Description:**
            - Sets or returns the email SMTP server DNS name for the common server setup information
              that is shared between the Act on Event commands and the Hardcopy Email commands.

        **Usage:**
            - Using the ``.query()`` method will send the ``EMAIL:SETUp:SMTPServer?`` query.
            - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp:SMTPServer?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMAIL:SETUp:SMTPServer value``
              command.

        **SCPI Syntax:**

        ::

            - EMAIL:SETUp:SMTPServer <QString>
            - EMAIL:SETUp:SMTPServer?
        """
        return self._smtpserver


class Email(SCPICmdRead):
    """The ``EMAIL`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``EMAIL?`` query.
        - Using the ``.verify(value)`` method will send the ``EMAIL?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.setup``: The ``EMAIL:SETUp`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "EMAIL") -> None:
        super().__init__(device, cmd_syntax)
        self._setup = EmailSetup(device, f"{self._cmd_syntax}:SETUp")

    @property
    def setup(self) -> EmailSetup:
        """Return the ``EMAIL:SETUp`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``EMAIL:SETUp?`` query.
            - Using the ``.verify(value)`` method will send the ``EMAIL:SETUp?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.fromaddress``: The ``EMAIL:SETUp:FROMADDRess`` command.
            - ``.hostaliasname``: The ``EMAIL:SETUp:HOSTALIASNAMe`` command.
            - ``.smtplogin``: The ``EMAIL:SETUp:SMTPLOGIn`` command.
            - ``.smtppassword``: The ``EMAIL:SETUp:SMTPPASSWord`` command.
            - ``.smtpport``: The ``EMAIL:SETUp:SMTPPort`` command.
            - ``.smtpserver``: The ``EMAIL:SETUp:SMTPServer`` command.
        """
        return self._setup
